@@ -29,9 +29,7 @@
 #include "errors.h"
 #include "prefs.h"
 
-#ifdef HAVE_COMPOSITE_EXTENSIONS
 #include "compositor.h"
-#endif
 
 #include <X11/Xatom.h>
 #include <string.h>
@@ -435,19 +433,7 @@ meta_workspace_queue_calc_showing  (MetaWorkspace *workspace)
   tmp = workspace->windows;
   while (tmp != NULL)
     {
-      if (meta_prefs_get_live_hidden_windows ())
-        {
-          /*
-           * When we hide rather than unmap windows, we need the show/hide
-           * status of the window to be recalculated *before* we call the
-           * compositor switch_workspace hook.
-           */
-          meta_window_calc_showing (tmp->data);
-        }
-      else
-        {
-          meta_window_queue (tmp->data, META_QUEUE_CALC_SHOWING);
-        }
+      meta_window_queue (tmp->data, META_QUEUE_CALC_SHOWING);
 
       tmp = tmp->next;
     }
@@ -458,8 +444,14 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
                                     MetaWindow    *focus_this,
                                     guint32        timestamp)
 {
-  MetaWorkspace *old;
-  MetaWindow *move_window;
+  MetaWorkspace  *old;
+  MetaWindow     *move_window;
+  MetaScreen     *screen;
+  MetaDisplay    *display;
+  MetaCompositor *comp;
+  MetaWorkspaceLayout layout1, layout2;
+  gint num_workspaces, current_space, new_space;
+  MetaMotionDirection direction;
   
   meta_verbose ("Activating workspace %d\n",
                 meta_workspace_index (workspace));
@@ -469,7 +461,7 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
 
   /* Note that old can be NULL; e.g. when starting up */
   old = workspace->screen->active_workspace;
-  
+
   workspace->screen->active_workspace = workspace;
 
   set_active_space_hint (workspace->screen);
@@ -516,6 +508,58 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
       /* Removes window from other spaces */
       meta_window_change_workspace (move_window, workspace);
 
+   /*
+    * Notify the compositor that the active workspace is changing.
+    */
+   screen = workspace->screen;
+   display = meta_screen_get_display (screen);
+   comp = meta_display_get_compositor (display);
+   direction = 0;
+
+   current_space = meta_workspace_index (old);
+   new_space     = meta_workspace_index (workspace);
+   num_workspaces = meta_screen_get_n_workspaces (workspace->screen);
+   meta_screen_calc_workspace_layout (workspace->screen, num_workspaces,
+                                      current_space, &layout1);
+
+   meta_screen_calc_workspace_layout (workspace->screen, num_workspaces,
+                                      new_space, &layout2);
+
+   if (layout1.current_col < layout2.current_col)
+     direction = META_MOTION_RIGHT;
+   if (layout1.current_col > layout2.current_col)
+     direction = META_MOTION_LEFT;
+
+   if (layout1.current_row < layout2.current_row)
+     {
+       if (!direction)
+         direction = META_MOTION_DOWN;
+       else if (direction == META_MOTION_RIGHT)
+         direction = META_MOTION_DOWN_RIGHT;
+       else
+         direction = META_MOTION_DOWN_LEFT;
+     }
+
+   if (layout1.current_row > layout2.current_row)
+     {
+       if (!direction)
+         direction = META_MOTION_UP;
+       else if (direction == META_MOTION_RIGHT)
+         direction = META_MOTION_UP_RIGHT;
+       else
+         direction = META_MOTION_UP_LEFT;
+     }
+
+   meta_screen_free_workspace_layout (&layout1);
+   meta_screen_free_workspace_layout (&layout2);
+
+   meta_compositor_switch_workspace (comp, screen, old, workspace, direction);
+
+  /* This needs to be done after telling the compositor we are switching
+   * workspaces since focusing a window will cause it to be immediately
+   * shown and that would confuse the compositor if it didn't know we
+   * were in a workspace switch.
+   */
   if (focus_this)
     {
       meta_window_focus (focus_this, timestamp);
@@ -531,59 +575,8 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
       meta_workspace_focus_default_window (workspace, NULL, timestamp);
     }
 
-#ifdef HAVE_COMPOSITE_EXTENSIONS
-  {
-    /*
-     * Notify the compositor that the active workspace changed.
-     */
-    MetaScreen     *screen = workspace->screen;
-    MetaDisplay    *display = meta_screen_get_display (screen);
-    MetaCompositor *comp = meta_display_get_compositor (display);
-    MetaWorkspaceLayout layout1, layout2;
-    gint num_workspaces, current_space, new_space;
-    MetaMotionDirection direction = 0;
-
-    current_space = meta_workspace_index (old);
-    new_space     = meta_workspace_index (workspace);
-
-    num_workspaces = meta_screen_get_n_workspaces (workspace->screen);
-    meta_screen_calc_workspace_layout (workspace->screen, num_workspaces,
-                                       current_space, &layout1);
-
-    meta_screen_calc_workspace_layout (workspace->screen, num_workspaces,
-                                       new_space, &layout2);
-
-    if (layout1.current_col < layout2.current_col)
-      direction = META_MOTION_RIGHT;
-    if (layout1.current_col > layout2.current_col)
-      direction = META_MOTION_LEFT;
-
-    if (layout1.current_row < layout2.current_row)
-      {
-        if (!direction)
-          direction = META_MOTION_DOWN;
-        else if (direction == META_MOTION_RIGHT)
-          direction = META_MOTION_DOWN_RIGHT;
-        else
-          direction = META_MOTION_DOWN_LEFT;
-      }
-
-    if (layout1.current_row > layout2.current_row)
-      {
-        if (!direction)
-          direction = META_MOTION_UP;
-        else if (direction == META_MOTION_RIGHT)
-          direction = META_MOTION_UP_RIGHT;
-        else
-          direction = META_MOTION_UP_LEFT;
-      }
-
-    meta_screen_free_workspace_layout (&layout1);
-    meta_screen_free_workspace_layout (&layout2);
-
-    meta_compositor_switch_workspace (comp, screen, old, workspace, direction);
-  }
-#endif
+   /* Emit switched signal from screen.c */
+   meta_screen_workspace_switched (screen, current_space, new_space, direction);
 }
 
 void
@@ -620,8 +613,14 @@ meta_workspace_update_window_hints (MetaWorkspace *workspace)
     }
 }
 
-/* get windows contained on workspace, including workspace->windows
- * and also sticky windows.
+/**
+ * meta_workspace_list_windows:
+ * @display: a #MetaDisplay
+ *
+ * Gets windows contained on the workspace, including workspace->windows
+ * and also sticky windows. Override-redirect windows are not included.
+ *
+ * Return value: (transfer container): the list of windows.
  */
 GList*
 meta_workspace_list_windows (MetaWorkspace *workspace)
@@ -630,7 +629,8 @@ meta_workspace_list_windows (MetaWorkspace *workspace)
   GSList *tmp;
   GList *workspace_windows;
   
-  display_windows = meta_display_list_windows (workspace->screen->display);
+  display_windows = meta_display_list_windows (workspace->screen->display,
+                                               META_LIST_DEFAULT);
 
   workspace_windows = NULL;
   tmp = display_windows;
@@ -919,7 +919,6 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
   /* We're all done, YAAY!  Record that everything has been validated. */
   workspace->work_areas_invalid = FALSE;
 
-#ifdef HAVE_COMPOSITE_EXTENSIONS
   {
     /*
      * Notify the compositor that the workspace geometry has changed.
@@ -928,9 +927,9 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
     MetaDisplay    *display = meta_screen_get_display (screen);
     MetaCompositor *comp = meta_display_get_compositor (display);
 
-    meta_compositor_update_workspace_geometry (comp, workspace);
+    if (comp)
+      meta_compositor_update_workspace_geometry (comp, workspace);
   }
-#endif
 }
 
 /**
